diff --git a/ChangeLog b/ChangeLog
index 598af20..7b9c271 100644
--- a/ChangeLog
+++ b/ChangeLog
@@ -1,3 +1,15 @@
+2007-09-12  brian m. carlson  <sandals@crustytoothpaste.ath.cx>
+
+	* src/cleanup.c (cu_prermremove): Set status to half-configured
+	when abort-remove fails.
+
+2007-09-12  brian m. carlson  <sandals@crustytoothpaste.ath.cx>
+
+	* src/remove.c (deferred_remove): Save old package status when
+	removing packages.
+	* src/cleanup.c (cu_prermremove): Restore old package status when
+	removal fails.
+
 2007-09-08  Guillem Jover  <guillem@debian.org>
 
 	* scripts/dpkg-buildpackage.sh: Use fakeroot, if present, as default
diff --git a/src/cleanup.c b/src/cleanup.c
index 108ee2c..01829af 100644
--- a/src/cleanup.c
+++ b/src/cleanup.c
@@ -233,12 +233,13 @@ void cu_postrmupgrade(int argc, void **argv) {
 
 void cu_prermremove(int argc, void **argv) {
   struct pkginfo *pkg= (struct pkginfo*)argv[0];
+  enum pkgstatus *oldpkgstatus= (enum pkgstatus *)argv[1];
 
-  if (cleanup_pkg_failed++) return;
+  if (cleanup_pkg_failed++) { pkg->status= stat_halfconfigured; return; }
+  pkg->status= *oldpkgstatus;
+  pkg->eflag &= ~eflagf_reinstreq;
   maintainer_script_installed(pkg,POSTINSTFILE,"post-installation",
                               "abort-remove", NULL);
-  pkg->status= stat_installed;
-  pkg->eflag &= ~eflagf_reinstreq;
   modstatdb_note(pkg);
   cleanup_pkg_failed--;
 }
diff --git a/src/remove.c b/src/remove.c
index abcfc9f..b5f10fa 100644
--- a/src/remove.c
+++ b/src/remove.c
@@ -84,9 +84,11 @@ void deferred_remove(struct pkginfo *pkg) {
   struct varbuf raemsgs;
   int rok;
   struct dependency *dep;
+  static enum pkgstatus oldpkgstatus;
 
   debug(dbg_general,"deferred_remove package %s",pkg->name);
   
+  oldpkgstatus= pkg->status;
   if (pkg->status == stat_notinstalled) {
     fprintf(stderr,
             _("dpkg - warning: ignoring request to remove %.250s which isn't installed.\n"),
@@ -168,7 +170,8 @@ void deferred_remove(struct pkginfo *pkg) {
     if (pkg->status == stat_installed || pkg->status == stat_halfconfigured) {
       pkg->status= stat_halfconfigured;
       modstatdb_note(pkg);
-      push_cleanup(cu_prermremove,~ehflag_normaltidy, 0,0, 1,(void*)pkg);
+      push_cleanup(cu_prermremove,~ehflag_normaltidy, 0,0, 2,(void*)pkg,
+                   (void *)&oldpkgstatus);
       maintainer_script_installed(pkg, PRERMFILE, "pre-removal",
                                   "remove", NULL);
     }
